@@ -12,7 +12,7 @@ import rapidjson
 import sqlalchemy as sa
 from sqlalchemy.engine.base import Engine
 
-from ..event import Event, EventKind
+from aionostr.event import Event, EventKind
 from ..config import Config
 from ..verification import Verifier
 from ..auth import get_authenticator, Action
@@ -49,18 +49,20 @@ class BaseStorage:
 class DBStorage(BaseStorage):
 
     def __init__(self, options):
-        self.options = options
-        self.db_url = options.get('sqlalchemy.url', 'sqlite+aiosqlite:///nostr.sqlite3')
+        self.options = options.copy()
+        self.db_url = self.options.pop('sqlalchemy.url', 'sqlite+aiosqlite:///nostr.sqlite3')
         self.clients = collections.defaultdict(dict)
         self.db = None
         self.garbage_collector_task = None
         self.is_postgres = 'postgresql' in self.db_url
         self._my_token = secrets.token_hex(4)
         self.log = logging.getLogger(__name__)
-        if self.is_postgres:
-            # sa.event.listen(Engine, "connect", self._subscribe_to_channel)
-            pass
-        else:
+
+        # limit the amount of concurrent inserts/selects
+        self.add_slot = asyncio.Semaphore(int(self.options.pop('num_concurrent_adds', 4)))
+        self.query_slot = asyncio.Semaphore(int(self.options.pop('num_concurrent_reqs', 10)))
+
+        if not self.is_postgres:
             # add event listener to set appropriate PRAGMA items
             sa.event.listen(Engine, "connect", self._set_sqlite_pragma)
 
@@ -80,17 +82,6 @@ class DBStorage(BaseStorage):
                 cursor.execute(stmt)
         cursor.close()
 
-    def _subscribe_to_channel(self, dbapi_connection, connection_record):
-        def add_listener():
-            asyncio.create_task(dbapi_connection.driver_connection.add_listener('addevent', self._event_listener))
-        asyncio.get_event_loop().call_later(2.0, add_listener)
-
-    async def _event_listener(self, connection, serverpid, channel, payload):
-        token, event_id = payload.split(':')
-        if token != self._my_token:
-            event_json = await self.get_event(event_id)
-            self._notify_all(Event(**event_json))
-
     async def close(self):
         if self.garbage_collector_task:
             self.garbage_collector_task.cancel()
@@ -104,19 +95,19 @@ class DBStorage(BaseStorage):
                 await conn.execute(sa.text("PRAGMA optimize"))
 
     async def setup(self):
-        # self.log.info("Database URL: '%s'", self.db_url)
         from sqlalchemy.ext.asyncio import create_async_engine
 
         self.authenticator = get_authenticator(self, Config.get('authentication', {}))
 
         self.verifier = Verifier(self, Config.get('verification', {}))
-
         self.db = create_async_engine(
             self.db_url,
-            future=True,
             json_deserializer=rapidjson.loads,
             pool_pre_ping=True,
+            **self.options
         )
+        self.log.info("Connected to %s", self.db.url)
+        self.loop = asyncio.get_running_loop()
 
         self.garbage_collector_task = start_garbage_collector(self.db)
         await self.verifier.start(self.db)
@@ -133,17 +124,24 @@ class DBStorage(BaseStorage):
             self.tag_insert_query = sa.insert(TagTable).prefix_with('OR IGNORE')
             self.event_insert_query = sa.insert(self.EventTable).prefix_with('OR IGNORE')
 
+        if Config.should_run_notifier:
+            from nostr_relay.notifier import NotifyClient
+
+            self.notifier = NotifyClient(self)
+            self.notifier.start()
+        else:
+            self.notifier = None
         self.log.debug("done setting up")
 
     async def get_event(self, event_id):
         """
         Shortcut for retrieving an event by id
         """
-        async with self.db.begin() as conn:
+        async with self.db.connect() as conn:
             result = await conn.execute(sa.select(self.EventTable).where(self.EventTable.c.id == bytes.fromhex(event_id)))
             row = result.first()
             if row:
-                return Event.from_tuple(row).to_json_object()
+                return Event.from_tuple(row)
 
     async def add_event(self, event_json, auth_token=None):
         """
@@ -156,31 +154,38 @@ class DBStorage(BaseStorage):
             self.log.error("bad json")
             raise StorageError("invalid: Bad JSON")
 
-        await asyncio.get_running_loop().run_in_executor(None, self.validate_event, event)
+        await self.loop.run_in_executor(None, self.validate_event, event)
         # check authentication
         if not await self.authenticator.can_do(auth_token, Action.save.value, event):
             raise AuthenticationError("restricted: permission denied")
 
         changed = False
-        async with self.db.begin() as conn:
-            do_save = await self.pre_save(conn, event)
-            if do_save:
-                result = await conn.execute(self.event_insert_query.values(
-                    id=event.id_bytes,
-                    created_at=event.created_at,
-                    pubkey=bytes.fromhex(event.pubkey),
-                    sig=bytes.fromhex(event.sig),
-                    content=event.content,
-                    kind=event.kind,
-                    tags=event.tags,
-                ))
-                changed = result.rowcount == 1
-                await self.post_save(conn, event, changed)
+        async with self.add_slot:
+            async with self.db.begin() as conn:
+                do_save = await self.pre_save(conn, event)
+                if do_save:
+                    result = await conn.execute(self.event_insert_query.values(
+                        id=event.id_bytes,
+                        created_at=event.created_at,
+                        pubkey=bytes.fromhex(event.pubkey),
+                        sig=bytes.fromhex(event.sig),
+                        content=event.content,
+                        kind=event.kind,
+                        tags=event.tags,
+                    ))
+                    changed = result.rowcount == 1
+                    await self.post_save(conn, event, changed)
         if changed:
-            self._notify_all(event)
+            self.notify_all_connected(event)
+            # notify other processes
+            await self.notify_other_processes(event)
         return event, changed
 
-    def _notify_all(self, event):
+    async def notify_other_processes(self, event):
+        if self.notifier:
+            await self.notifier.notify(event)
+
+    def notify_all_connected(self, event):
         # notify all subscriptions
         count = 0
         with catchtime() as t:
@@ -318,7 +323,7 @@ class DBStorage(BaseStorage):
         queue = asyncio.Queue()
         sub = Subscription(self.db, '', query_filters, queue=queue, default_limit=600000, log=self.log)
         sub.prepare()
-        await sub.run_query()
+        await sub.run_query(self.query_slot)
         while True:
             _, event = await queue.get()
             if event is None:
@@ -346,7 +351,7 @@ class DBStorage(BaseStorage):
             if not await self.authenticator.can_do(auth_token, Action.query.value, sub):
                 raise AuthenticationError("restricted: permission denied")
 
-            asyncio.create_task(sub.run_query())
+            asyncio.create_task(sub.run_query(self.query_slot))
             self.clients[client_id][sub_id] = sub
             self.log.debug("%s/%s +", client_id, sub_id)
 
@@ -466,7 +471,7 @@ class Subscription:
         self.default_limit = default_limit
         self.is_postgres = is_postgres
         self.log = log
-        self.long_query_threshold = 500
+        self.long_query_threshold = 1000
 
     def prepare(self):
         try:
@@ -480,8 +485,7 @@ class Subscription:
         if self.query_task:
             self.query_task.cancel()
 
-    async def run_query(self):
-        self.query_task = asyncio.current_task()
+    async def run_query(self, query_slot: asyncio.Semaphore):
 
         query = self.query
         self.log.debug(query)
@@ -489,14 +493,16 @@ class Subscription:
         queue = self.queue
         try:
             count = 0
-            with catchtime() as t:
-                async with self.db.connect() as conn:
-                    result = await conn.stream(sa.text(query))
-                    async for row in result:
-                        event = Event.from_tuple(row)
-                        await queue.put((sub_id, event))
-                        count += 1
-                await queue.put((sub_id, None))
+            async with query_slot:
+                with catchtime() as t:
+                    query = sa.text(query)
+                    async with self.db.connect() as conn:
+                        async with conn.stream(query) as result:
+                            async for row in result:
+                                event = Event.from_tuple(row)
+                                await queue.put((sub_id, event))
+                                count += 1
+                    await queue.put((sub_id, None))
 
             duration = t()
             self.log.info('%s/%s query – events:%s duration:%dms', self.client_id, self.sub_id, count, duration)
