@@ -50,7 +50,7 @@ class Client:
             try:
                 sub_id, event = await subscription_queue.get()
                 if event is not None:
-                    message = f'["EVENT", "{sub_id}", {event}]'
+                    message = event.to_message(sub_id)
                 else:
                     # done with stored events
                     message = f'["EOSE", "{sub_id}"]'
@@ -239,7 +239,7 @@ class ViewEventResource(BaseResource):
         except Exception:
             self.log.exception('get-event')
         if event:
-            resp.media = event
+            resp.media = event.to_json_object()
         else:
             raise falcon.HTTPNotFound
 
@@ -270,9 +270,12 @@ class SetupMiddleware:
         self.storage = storage
 
     async def process_startup(self, scope, event):
-        import random
         if Config.DEBUG:
             asyncio.get_running_loop().set_debug(True)
+        if Config.should_run_notifier:
+            from .notifier import NotifyServer
+            notify_server = NotifyServer()
+            notify_server.start()
         await self.storage.setup()
 
     async def process_shutdown(self, scope, event):
